@@ -49,18 +49,10 @@
 #include "ev-attachment.h"
 #include "ev-image.h"
 
-#if (defined (HAVE_POPPLER_PAGE_RENDER) || defined (HAVE_POPPLER_PAGE_RENDER_FOR_PRINTING)) && (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
+#if (defined (HAVE_POPPLER_PAGE_RENDER)) && (defined (HAVE_CAIRO_PDF) || defined (HAVE_CAIRO_PS))
 #define HAVE_CAIRO_PRINT
 #endif
 
-#if POPPLER_MAJOR_VERSION <= 6 || (POPPLER_MAJOR_VERSION == 7 && POPPLER_MINOR_VERSION < 2)
-#define POPPLER_HAS_GDK
-#else
-#ifdef POPPLER_WITH_GDK
-#define POPPLER_HAS_GDK
-#endif
-#endif
-
 typedef struct {
 	PdfDocument *document;
 	char *text;
@@ -156,19 +148,6 @@ EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 								 pdf_document_page_transition_iface_init);
 			 });
 
-static void
-set_rc_data (PdfDocument     *pdf_document,
-	     EvRenderContext *rc)
-{
-	if (rc->data == NULL) {
-		rc->data = poppler_document_get_page (pdf_document->document,
-						      rc->page);
-		rc->destroy = g_object_unref;
-	} else {
-		g_assert (rc->page == poppler_page_get_index (POPPLER_PAGE (rc->data)));
-	}
-}
-
 static void
 pdf_document_search_free (PdfDocumentSearch   *search)
 {
@@ -304,35 +283,45 @@ pdf_document_get_n_pages (EvDocument *document)
 	return poppler_document_get_n_pages (PDF_DOCUMENT (document)->document);
 }
 
-static void
-pdf_document_get_page_size (EvDocument   *document,
-			    int           page,
-			    double       *width,
-			    double       *height)
+static EvPage *
+pdf_document_get_page (EvDocument *document,
+		       gint        index)
 {
 	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerPage *poppler_page;
+	EvPage      *page;
 
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	poppler_page_get_size (poppler_page, width, height);
+	poppler_page = poppler_document_get_page (pdf_document->document, index);
+	page = ev_page_new (index);
+	page->backend_page = (EvBackendPage)g_object_ref (poppler_page);
+	page->backend_destroy_func = (EvBackendPageDestroyFunc)g_object_unref;
 	g_object_unref (poppler_page);
+
+	return page;
+}
+
+static void
+pdf_document_get_page_size (EvDocument *document,
+			    EvPage     *page,
+			    double     *width,
+			    double     *height)
+{
+	g_return_if_fail (POPPLER_IS_PAGE (page->backend_page));
+	
+	poppler_page_get_size (POPPLER_PAGE (page->backend_page), width, height);
 }
 
 static char *
 pdf_document_get_page_label (EvDocument *document,
-			     int         page)
+			     EvPage     *page)
 {
-	PopplerPage *poppler_page;
 	char *label = NULL;
 
-	poppler_page = poppler_document_get_page (PDF_DOCUMENT (document)->document,
-						  page);
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
 
-	g_object_get (G_OBJECT (poppler_page),
+	g_object_get (G_OBJECT (page->backend_page),
 		      "label", &label,
 		      NULL);
-	g_object_unref (poppler_page);
-
 	return label;
 }
 
@@ -512,14 +501,13 @@ pdf_document_render (EvDocument      *document,
 		     EvRenderContext *rc)
 {
 	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
 	double width_points, height_points;
 	gint width, height;
 
-	pdf_document = PDF_DOCUMENT (document);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
-	set_rc_data (pdf_document, rc);
-
-	poppler_page_get_size (POPPLER_PAGE (rc->data),
+	poppler_page_get_size (poppler_page,
 			       &width_points, &height_points);
 	
 	if (rc->rotation == 90 || rc->rotation == 270) {
@@ -530,7 +518,7 @@ pdf_document_render (EvDocument      *document,
 		height = (int) ((height_points * rc->scale) + 0.5);
 	}
 	
-	return pdf_page_render (POPPLER_PAGE (rc->data),
+	return pdf_page_render (poppler_page,
 				width, height, rc);
 }
 
@@ -563,6 +551,7 @@ pdf_document_get_info (EvDocument *document)
 	PopplerPageMode mode;
 	PopplerViewerPreferences view_prefs;
 	PopplerPermissions permissions;
+	EvPage *page;
 
 	info = g_new0 (EvDocumentInfo, 1);
 
@@ -601,13 +590,20 @@ pdf_document_get_info (EvDocument *document)
 		      "linearized", &(info->linearized),
 		      NULL);
 
-	pdf_document_get_page_size(document, 0,
-				   &(info->paper_width),
-				   &(info->paper_height));
+	info->n_pages = ev_document_get_n_pages (document);
 
-	// Convert to mm.
-	info->paper_width = info->paper_width / 72.0f * 25.4f;
-	info->paper_height = info->paper_height / 72.0f * 25.4f;
+	if (info->n_pages > 0) {
+		page = ev_document_get_page (document, 0);
+		ev_document_get_page_size (document, page,
+					   &(info->paper_width),
+					   &(info->paper_height));
+		g_object_unref (page);
+		
+
+		// Convert to mm.
+		info->paper_width = info->paper_width / 72.0f * 25.4f;
+		info->paper_height = info->paper_height / 72.0f * 25.4f;
+	}
 
 	switch (layout) {
 		case POPPLER_PAGE_LAYOUT_SINGLE_PAGE:
@@ -687,8 +683,6 @@ pdf_document_get_info (EvDocument *document)
 		info->permissions |= EV_DOCUMENT_PERMISSIONS_OK_TO_ADD_NOTES;
 	}
 
-	info->n_pages = ev_document_get_n_pages (document);
-
 	if (ev_document_security_has_document_security (EV_DOCUMENT_SECURITY (document))) {
 		/* translators: this is the document security state */
 		info->security = g_strdup (_("Yes"));
@@ -706,6 +700,7 @@ pdf_document_document_iface_init (EvDocumentIface *iface)
 	iface->save = pdf_document_save;
 	iface->load = pdf_document_load;
 	iface->get_n_pages = pdf_document_get_n_pages;
+	iface->get_page = pdf_document_get_page;
 	iface->get_page_size = pdf_document_get_page_size;
 	iface->get_page_label = pdf_document_get_page_label;
 	iface->has_attachments = pdf_document_has_attachments;
@@ -960,6 +955,8 @@ ev_link_from_action (PdfDocument   *pdf_document,
 	const char   *unimplemented_action = NULL;
 
 	switch (action->type) {
+	        case POPPLER_ACTION_NONE:
+			break;
 	        case POPPLER_ACTION_GOTO_DEST: {
 			EvLinkDest *dest;
 			
@@ -1197,11 +1194,8 @@ pdf_document_images_get_image_mapping (EvDocumentImages *document_images,
 		image_mapping = (PopplerImageMapping *)list->data;
 
 		ev_image_mapping = g_new (EvImageMapping, 1);
-#ifdef HAVE_POPPLER_PAGE_GET_IMAGE
+		
 		ev_image_mapping->image = ev_image_new (page, image_mapping->image_id);
-#elif POPPLER_HAS_CAIRO
-		ev_image_mapping->image = ev_image_new_from_pixbuf (image_mapping->image);
-#endif
 		ev_image_mapping->x1 = image_mapping->area.x1;
 		ev_image_mapping->x2 = image_mapping->area.x2;
 		ev_image_mapping->y1 = image_mapping->area.y1;
@@ -1261,7 +1255,7 @@ make_thumbnail_for_page (PdfDocument     *pdf_document,
 
 	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document),
 						rc, &width, &height);
-#ifdef POPPLER_HAS_GDK
+#ifdef POPPLER_WITH_GDK
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
 				 width, height);
 	gdk_pixbuf_fill (pixbuf, 0xffffffff);
@@ -1280,7 +1274,7 @@ make_thumbnail_for_page (PdfDocument     *pdf_document,
 	
 	pixbuf = ev_document_misc_pixbuf_from_surface (surface);
 	cairo_surface_destroy (surface);
-#endif /* POPPLER_HAS_GDK */
+#endif /* POPPLER_WITH_GDK */
 
 	return pixbuf;
 }
@@ -1290,20 +1284,14 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 				       EvRenderContext      *rc, 
 				       gboolean              border)
 {
-	PdfDocument *pdf_document;
+	PdfDocument *pdf_document = PDF_DOCUMENT (document_thumbnails);
 	PopplerPage *poppler_page;
 	GdkPixbuf *pixbuf = NULL;
 	GdkPixbuf *border_pixbuf;
 
-	pdf_document = PDF_DOCUMENT (document_thumbnails);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-	g_return_val_if_fail (poppler_page != NULL, NULL);
-
-#if POPPLER_MAJOR_VERSION <= 6 || (POPPLER_MAJOR_VERSION == 7 && POPPLER_MINOR_VERSION < 2)
-	pixbuf = poppler_page_get_thumbnail (poppler_page);
-#else
-#ifdef POPPLER_HAS_GDK
+#ifdef POPPLER_WITH_GDK
 	pixbuf = poppler_page_get_thumbnail_pixbuf (poppler_page);
 #else
 	cairo_surface_t *surface;
@@ -1313,9 +1301,8 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 		pixbuf = ev_document_misc_pixbuf_from_surface (surface);
 		cairo_surface_destroy (surface);
 	}
-#endif
-#endif
-		
+#endif /* POPPLER_WITH_GDK */
+
 	if (pixbuf) {
 		/* Rotate provided thumbnail if needed */
 		GdkPixbuf *rotated_pixbuf;
@@ -1335,8 +1322,6 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 		pixbuf = border_pixbuf;
 	}		
 
-	g_object_unref (poppler_page);
-	
 	return pixbuf;
 }
 
@@ -1346,18 +1331,14 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 					gint                 *width,
 					gint                 *height)
 {
-	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
 	gint has_thumb;
 	
-	pdf_document = PDF_DOCUMENT (document_thumbnails);
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-
-	g_return_if_fail (poppler_page != NULL);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	has_thumb = poppler_page_get_thumbnail_size (poppler_page, width, height);
 
-	if (!has_thumb) {
+	if (!has_thumb || *width <= 0 || *height <= 0) {
 		double page_width, page_height;
 
 		poppler_page_get_size (poppler_page, &page_width, &page_height);
@@ -1373,8 +1354,6 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 		*width = *height;
 		*height = temp;
 	}
-	
-	g_object_unref (poppler_page);
 }
 
 static void
@@ -1385,195 +1364,42 @@ pdf_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface)
 }
 
 
-static gboolean
-pdf_document_search_idle_callback (void *data)
+static GList *
+pdf_document_find_find_text (EvDocumentFind *document_find,
+			     EvPage         *page,
+			     const gchar    *text,
+			     gboolean        case_sensitive)
 {
-        PdfDocumentSearch *search = (PdfDocumentSearch*) data;
-        PdfDocument *pdf_document = search->document;
-        int n_pages;
-	GList *matches;
-	PopplerPage *page;
-
-	page = poppler_document_get_page (search->document->document,
-					  search->search_page);
-
-	ev_document_doc_mutex_lock ();
-	matches = poppler_page_find_text (page, search->text);
-	ev_document_doc_mutex_unlock ();
-
-	g_object_unref (page);
-
-	search->pages[search->search_page] = matches;
-	ev_document_find_changed (EV_DOCUMENT_FIND (pdf_document),
-				  search->search_page);
-
-        n_pages = pdf_document_get_n_pages (EV_DOCUMENT (search->document));
-        search->search_page += 1;
-        if (search->search_page == n_pages) {
-                /* wrap around */
-                search->search_page = 0;
-        }
-
-        if (search->search_page != search->start_page) {
-	        return TRUE;
-	}
-
-        /* We're done. */
-        search->idle = 0; /* will return FALSE to remove */
-        return FALSE;
-}
-
-
-static PdfDocumentSearch *
-pdf_document_search_new (PdfDocument *pdf_document,
-			 int          start_page,
-			 const char  *text)
-{
-	PdfDocumentSearch *search;
-	int n_pages;
-	int i;
-
-	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (pdf_document));
-
-        search = g_new0 (PdfDocumentSearch, 1);
-
-	search->text = g_strdup (text);
-        search->pages = g_new0 (GList *, n_pages);
-        search->document = pdf_document;
-
-        /* We add at low priority so the progress bar repaints */
-        search->idle = g_idle_add_full (G_PRIORITY_LOW,
-                                        pdf_document_search_idle_callback,
-                                        search,
-                                        NULL);
-
-        search->start_page = start_page;
-        search->search_page = start_page;
-
-	return search;
-}
-
-static void
-pdf_document_find_begin (EvDocumentFind   *document,
-			 int               page,
-                         const char       *search_string,
-                         gboolean          case_sensitive)
-{
-        PdfDocument *pdf_document = PDF_DOCUMENT (document);
-
-        /* FIXME handle case_sensitive (right now XPDF
-         * code is always case insensitive for ASCII
-         * and case sensitive for all other languaages)
-         */
-
-	if (pdf_document->search &&
-	    strcmp (search_string, pdf_document->search->text) == 0)
-                return;
-
-        if (pdf_document->search)
-                pdf_document_search_free (pdf_document->search);
-
-        pdf_document->search = pdf_document_search_new (pdf_document,
-							page,
-							search_string);
-}
-
-static int
-pdf_document_find_get_n_results (EvDocumentFind *document_find, int page)
-{
-	PdfDocumentSearch *search = PDF_DOCUMENT (document_find)->search;
-
-	if (search) {
-		return g_list_length (search->pages[page]);
-	} else {
-		return 0;
-	}
-}
-
-static gboolean
-pdf_document_find_get_result (EvDocumentFind *document_find,
-			      int             page,
-			      int             n_result,
-			      EvRectangle    *rectangle)
-{
-	PdfDocument *pdf_document = PDF_DOCUMENT (document_find);
-	PdfDocumentSearch *search = pdf_document->search;
+	GList *matches, *l;
 	PopplerPage *poppler_page;
-	PopplerRectangle *r;
-	double height;
+	gdouble height;
+	
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
 
-	if (search == NULL)
-		return FALSE;
+	poppler_page = POPPLER_PAGE (page->backend_page);
+	
+	matches = poppler_page_find_text (poppler_page, text);
+	if (!matches)
+		return NULL;
 
-	r = (PopplerRectangle *) g_list_nth_data (search->pages[page],
-						  n_result);
-	if (r == NULL)
-		return FALSE;
-
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
 	poppler_page_get_size (poppler_page, NULL, &height);
-	rectangle->x1 = r->x1;
-	rectangle->y1 = height - r->y2;
-	rectangle->x2 = r->x2;
-	rectangle->y2 = height - r->y1;
-	g_object_unref (poppler_page);
-		
-	return TRUE;
-}
+	for (l = matches; l && l->data; l = g_list_next (l)) {
+		PopplerRectangle *rect = (PopplerRectangle *)l->data;
+		gdouble           tmp;
 
-static int
-pdf_document_find_page_has_results (EvDocumentFind *document_find,
-				    int             page)
-{
-	PdfDocumentSearch *search = PDF_DOCUMENT (document_find)->search;
-
-	return search && search->pages[page] != NULL;
-}
-
-static double
-pdf_document_find_get_progress (EvDocumentFind *document_find)
-{
-	PdfDocumentSearch *search;
-	int n_pages, pages_done;
-
-	search = PDF_DOCUMENT (document_find)->search;
-
-	if (search == NULL) {
-		return 0;
-	}
-
-	n_pages = pdf_document_get_n_pages (EV_DOCUMENT (document_find));
-	if (search->search_page > search->start_page) {
-		pages_done = search->search_page - search->start_page + 1;
-	} else if (search->search_page == search->start_page) {
-		pages_done = n_pages;
-	} else {
-		pages_done = n_pages - search->start_page + search->search_page;
-	}
-
-	return pages_done / (double) n_pages;
-}
-
-static void
-pdf_document_find_cancel (EvDocumentFind *document)
-{
-        PdfDocument *pdf_document = PDF_DOCUMENT (document);
-
-	if (pdf_document->search) {
-		pdf_document_search_free (pdf_document->search);
-		pdf_document->search = NULL;
+		tmp = rect->y1;
+		rect->y1 = height - rect->y2;
+		rect->y2 = height - tmp;
 	}
+	
+	return matches;
 }
 
 static void
 pdf_document_find_iface_init (EvDocumentFindIface *iface)
 {
-        iface->begin = pdf_document_find_begin;
-	iface->get_n_results = pdf_document_find_get_n_results;
-	iface->get_result = pdf_document_find_get_result;
-	iface->page_has_results = pdf_document_find_page_has_results;
-	iface->get_progress = pdf_document_find_get_progress;
-        iface->cancel = pdf_document_find_cancel;
+        iface->find_text = pdf_document_find_find_text;
 }
 
 static void
@@ -1721,7 +1547,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 
 	g_return_if_fail (pdf_document->print_ctx != NULL);
 
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 	
 #ifdef HAVE_CAIRO_PRINT
 	x = (ctx->pages_printed % ctx->pages_per_sheet) % ctx->pages_x;
@@ -1798,13 +1624,7 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 			 y * (rotate ? pwidth : pheight));
 	cairo_scale (ctx->cr, xscale, yscale);
 
-#ifdef HAVE_POPPLER_PAGE_RENDER_FOR_PRINTING
 	poppler_page_render_for_printing (poppler_page, ctx->cr);
-#else
-#ifdef HAVE_POPPLER_PAGE_RENDER
-	poppler_page_render (poppler_page, ctx->cr);
-#endif
-#endif
 
 	ctx->pages_printed++;
 			
@@ -1813,8 +1633,6 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	if (ctx->format == EV_FILE_FORMAT_PS)
 		poppler_page_render_to_ps (poppler_page, ctx->ps_file);
 #endif /* HAVE_CAIRO_PRINT */
-	
-	g_object_unref (poppler_page);
 }
 
 static void
@@ -1885,27 +1703,19 @@ pdf_selection_render_selection (EvSelection      *selection,
 				GdkColor         *text,
 				GdkColor         *base)
 {
-	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
 	double width_points, height_points;
 	gint width, height;
 
-	pdf_document = PDF_DOCUMENT (selection);
-	set_rc_data (pdf_document, rc);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
-	poppler_page_get_size (POPPLER_PAGE (rc->data),
+	poppler_page_get_size (poppler_page,
 			       &width_points, &height_points);
 	width = (int) ((width_points * rc->scale) + 0.5);
 	height = (int) ((height_points * rc->scale) + 0.5);
 
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
-	
-#if POPPLER_MAJOR_VERSION <= 6 || (POPPLER_MAJOR_VERSION == 7 && POPPLER_MINOR_VERSION < 2)
-	GdkColor text_color, base_color;
-
-	text_color = *text;
-	base_color = *base;
-#else
 	PopplerColor text_color, base_color;
 	
 	text_color.red = text->red;
@@ -1915,7 +1725,6 @@ pdf_selection_render_selection (EvSelection      *selection,
 	base_color.red = base->red;
 	base_color.green = base->green;
 	base_color.blue = base->blue;
-#endif
 
 	if (*surface == NULL) {
 		*surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
@@ -1929,7 +1738,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 	memset (cairo_image_surface_get_data (*surface), 0x00,
 		cairo_image_surface_get_height (*surface) *
 		cairo_image_surface_get_stride (*surface));
-	poppler_page_render_selection (POPPLER_PAGE (rc->data),
+	poppler_page_render_selection (poppler_page,
 				       cr,
 				       (PopplerRectangle *)points,
 				       (PopplerRectangle *)old_points,
@@ -1944,7 +1753,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 				 TRUE, 8,
 				 width, height);
 
-	poppler_page_render_selection_to_pixbuf (POPPLER_PAGE (rc->data),
+	poppler_page_render_selection_to_pixbuf (poppler_page,
 						 rc->scale, rc->rotation, pixbuf,
 						 (PopplerRectangle *)points,
 						 (PopplerRectangle *)old_points,
@@ -1964,14 +1773,12 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 				 EvSelectionStyle style,
 				 EvRectangle     *points)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (selection);
 	PopplerPage *poppler_page;
 	PopplerRectangle r;
 	double height;
 	char *retval;
 	
-	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-	g_return_val_if_fail (poppler_page != NULL, NULL);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	poppler_page_get_size (poppler_page, NULL, &height);
 	r.x1 = points->x1;
@@ -1983,8 +1790,6 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 					(PopplerSelectionStyle)style,
 					&r);
 
-	g_object_unref (poppler_page);
-
 	return retval;
 }
 
@@ -2020,28 +1825,18 @@ pdf_selection_get_selection_region (EvSelection     *selection,
 				    EvSelectionStyle style,
 				    EvRectangle     *points)
 {
-	PdfDocument *pdf_document;
+	PopplerPage *poppler_page;
 	GdkRegion   *retval;
+	GList       *region;
 
-	pdf_document = PDF_DOCUMENT (selection);
-
-	set_rc_data (pdf_document, rc);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 	
-#if POPPLER_MAJOR_VERSION <= 6 || (POPPLER_MAJOR_VERSION == 7 && POPPLER_MINOR_VERSION < 2)
-	retval = poppler_page_get_selection_region (POPPLER_PAGE (rc->data),
-						    rc->scale,
-						    (PopplerSelectionStyle)style,
-						    (PopplerRectangle *) points);
-#else
-	GList *region;
-	
-	region = poppler_page_get_selection_region (POPPLER_PAGE (rc->data),
+	region = poppler_page_get_selection_region (poppler_page,
 						    rc->scale,
 						    (PopplerSelectionStyle)style,
 						    (PopplerRectangle *) points);
 	retval = create_gdk_region_from_poppler_region (region);
 	g_list_free (region);
-#endif
 	
 	return retval;
 }
@@ -2050,32 +1845,22 @@ static GdkRegion *
 pdf_selection_get_selection_map (EvSelection     *selection,
 				 EvRenderContext *rc)
 {
-	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
 	PopplerRectangle points;
 	GList *region;
 	GdkRegion *retval;
 
-	pdf_document = PDF_DOCUMENT (selection);
-	poppler_page = poppler_document_get_page (pdf_document->document,
-						  rc->page);
+	poppler_page = POPPLER_PAGE (rc->page->backend_page);
 
 	points.x1 = 0.0;
 	points.y1 = 0.0;
 	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
 	
-#if POPPLER_MAJOR_VERSION <= 6 || (POPPLER_MAJOR_VERSION == 7 && POPPLER_MINOR_VERSION < 2)
-	retval = poppler_page_get_selection_region (poppler_page, 1.0,
-						    POPPLER_SELECTION_GLYPH,
-						    &points);
-#else
 	region = poppler_page_get_selection_region (poppler_page, 1.0,
 						    POPPLER_SELECTION_GLYPH,
 						    &points);
 	retval = create_gdk_region_from_poppler_region (region);
 	g_list_free (region);
-#endif
-	g_object_unref (poppler_page);
 
 	return retval;
 }
@@ -2279,17 +2064,17 @@ ev_form_field_from_poppler_field (PopplerFormField *poppler_field)
 
 static GList *
 pdf_document_forms_get_form_fields (EvDocumentForms *document, 
-				    gint             page)
+				    EvPage          *page)
 {
- 	PdfDocument *pdf_document;
  	PopplerPage *poppler_page;
  	GList *retval = NULL;
  	GList *fields;
  	GList *list;
  	double height;
 
- 	pdf_document = PDF_DOCUMENT (document);
- 	poppler_page = poppler_document_get_page (pdf_document->document, page);
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+	
+ 	poppler_page = POPPLER_PAGE (page->backend_page);
  	fields = poppler_page_get_form_field_mapping (poppler_page);
  	poppler_page_get_size (poppler_page, NULL, &height);
 
@@ -2310,13 +2095,17 @@ pdf_document_forms_get_form_fields (EvDocumentForms *document,
 		field_mapping->y1 = height - mapping->area.y2;
 		field_mapping->y2 = height - mapping->area.y1;
 		field_mapping->field = ev_field;
-		field_mapping->field->page = page;
+		field_mapping->field->page = EV_PAGE (g_object_ref (page));
+
+		g_object_set_data_full (G_OBJECT (ev_field),
+					"poppler-field",
+					g_object_ref (mapping->field),
+					(GDestroyNotify) g_object_unref);
 		
 		retval = g_list_prepend (retval, field_mapping);
 	}
 	
 	poppler_page_free_form_field_mapping (fields);
-	g_object_unref (poppler_page);
 
 	return g_list_reverse (retval);
 }
@@ -2326,16 +2115,14 @@ pdf_document_forms_form_field_text_get_text (EvDocumentForms *document,
 					     EvFormField     *field)
 	
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 	
 	text = poppler_form_field_text_get_text (poppler_field);
-	g_object_unref (poppler_field);
 
 	return text;
 }
@@ -2345,14 +2132,12 @@ pdf_document_forms_form_field_text_set_text (EvDocumentForms *document,
 					     EvFormField     *field,
 					     const gchar     *text)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	poppler_form_field_text_set_text (poppler_field, text);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2360,31 +2145,27 @@ pdf_document_forms_form_field_button_set_state (EvDocumentForms *document,
 						EvFormField     *field,
 						gboolean         state)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_button_set_state (poppler_field, state);
-	g_object_unref (poppler_field);
 }
 
 static gboolean
 pdf_document_forms_form_field_button_get_state (EvDocumentForms *document, 
 						EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gboolean state;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return FALSE;
 
 	state = poppler_form_field_button_get_state (poppler_field);
-	g_object_unref (poppler_field);
 
 	return state;
 }
@@ -2394,16 +2175,14 @@ pdf_document_forms_form_field_choice_get_item (EvDocumentForms *document,
 					       EvFormField     *field,
 					       gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 
 	text = poppler_form_field_choice_get_item (poppler_field, index);
-	g_object_unref (poppler_field);
 
 	return text;
 }
@@ -2412,16 +2191,14 @@ static int
 pdf_document_forms_form_field_choice_get_n_items (EvDocumentForms *document, 
 						  EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gint n_items;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return -1;
 	
 	n_items = poppler_form_field_choice_get_n_items (poppler_field);
-	g_object_unref (poppler_field);
 
 	return n_items;
 }
@@ -2431,16 +2208,14 @@ pdf_document_forms_form_field_choice_is_item_selected (EvDocumentForms *document
 						       EvFormField     *field,
 						       gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gboolean selected;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return FALSE;
 
 	selected = poppler_form_field_choice_is_item_selected (poppler_field, index);
-	g_object_unref (poppler_field);
 
 	return selected;
 }
@@ -2450,15 +2225,13 @@ pdf_document_forms_form_field_choice_select_item (EvDocumentForms *document,
 						  EvFormField     *field,
 						  gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 
 	poppler_form_field_choice_select_item (poppler_field, index);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2466,30 +2239,26 @@ pdf_document_forms_form_field_choice_toggle_item (EvDocumentForms *document,
 						  EvFormField     *field,
 						  gint             index)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 
 	poppler_form_field_choice_toggle_item (poppler_field, index);
-	g_object_unref (poppler_field);
 }
 
 static void
 pdf_document_forms_form_field_choice_unselect_all (EvDocumentForms *document, 
 						   EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_choice_unselect_all (poppler_field);
-	g_object_unref (poppler_field);
 }
 
 static void
@@ -2497,31 +2266,27 @@ pdf_document_forms_form_field_choice_set_text (EvDocumentForms *document,
 					       EvFormField     *field,
 					       const gchar     *text)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return;
 	
 	poppler_form_field_choice_set_text (poppler_field, text);
-	g_object_unref (poppler_field);
 }
 
 static gchar *
 pdf_document_forms_form_field_choice_get_text (EvDocumentForms *document,
 					       EvFormField     *field)
 {
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
 	PopplerFormField *poppler_field;
 	gchar *text;
 
-	poppler_field = poppler_document_get_form_field (pdf_document->document, field->id);
+	poppler_field = POPPLER_FORM_FIELD (g_object_get_data (G_OBJECT (field), "poppler-field"));
 	if (!poppler_field)
 		return NULL;
 
 	text = poppler_form_field_choice_get_text (poppler_field);
-	g_object_unref (poppler_field);
 
 	return text;
 }
